package mk.ukim.finki.db.library.service.impl;

import mk.ukim.finki.db.library.model.*;
import mk.ukim.finki.db.library.model.exception.InvalidArgumentsException;
import mk.ukim.finki.db.library.model.exception.InvalidBookIdException;
import mk.ukim.finki.db.library.model.exception.LibraryNotFoundException;
import mk.ukim.finki.db.library.repository.BookRepository;
import mk.ukim.finki.db.library.repository.GenreRepository;
import mk.ukim.finki.db.library.repository.LibraryRepository;
import mk.ukim.finki.db.library.repository.WriterRepository;
import mk.ukim.finki.db.library.service.BookService;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class BookServiceImpl implements BookService {

    private final BookRepository bookRepository;
    private final WriterRepository writerRepository;
    private final GenreRepository genreRepository;
    private final LibraryRepository libraryRepository;

    public BookServiceImpl(BookRepository bookRepository, WriterRepository writerRepository, GenreRepository genreRepository, LibraryRepository libraryRepository) {
        this.bookRepository = bookRepository;
        this.writerRepository = writerRepository;
        this.genreRepository = genreRepository;
        this.libraryRepository = libraryRepository;
    }

    @Override
    public List<Book> listAllBooks() {
        return this.bookRepository.findAll();
    }

    @Override
    public Book findById(Long id) {
        return this.bookRepository.findById(id).orElseThrow(InvalidBookIdException::new);
    }

    @Override
    public Book create(Long id, String bookName, int bookNumber, boolean isFree, int bookPrice, int bookPriceLate) { //, List<Long> genreIds, List<Long> writerIds
        //List<Genre> genres = this.genreRepository.findAllById(genreIds);
        //List<Writer> writers = this.writerRepository.findAllById(writerIds);
        Book book = new Book(id, bookName, bookNumber, isFree, bookPrice, bookPriceLate); //, genres, writers
        return this.bookRepository.save(book);
    }

    @Override
    public Book update(Long id, String bookName, int bookNumber, boolean isFree, int bookPrice, int bookPriceLate, List<Long> genreIds, List<Long> writerIds, Long libraryId) {
        Book book =  this.findById(id);
        book.setBookName(bookName);
        book.setBookNumber(bookNumber);
        book.setFree(isFree);
        book.setBookPrice(bookPrice);
        book.setBookPriceLate(bookPriceLate);

        List<Genre> genres = this.genreRepository.findAllById(genreIds);
        List<Writer> writers = this.writerRepository.findAllById(genreIds);

        book.setGenre((Genre) genres);
        book.setWriters(writers);

        Library library = this.libraryRepository.findById(libraryId).orElseThrow(() -> new LibraryNotFoundException(libraryId));
        book.setLibrary(library);

        return this.bookRepository.save(book);
    }

    @Override
    public Book delete(Long id) {
        Book book =  this.findById(id);
        this.bookRepository.delete(book);
        return book;
    }

    @Override
    public Book createB(Long id, String bookName, int bookNumber, boolean isFree, int bookPrice, int bookPriceLate, List<Long> genreIds, List<Long> writerIds, Long libraryId) {
        List<Genre> genres = this.genreRepository.findAllById(genreIds);
        List<Writer> writers = this.writerRepository.findAllById(genreIds);

        Book book = new Book();
        book.setId(id);
        book.setBookName(bookName);
        book.setBookNumber(bookNumber);
        book.setFree(isFree);
        book.setBookPrice(bookPrice);
        book.setBookPriceLate(bookPriceLate);
        book.setGenre((Genre) genres);
        book.setWriters(writers);

        //book = new Book(bookName, bookNumber, isFree, bookPrice, bookPriceLate, genreIds, writerIds, libraryId);
        return null;
    }

    @Override
    public List<Book> findByBookName(String bookName) {
        if(bookName == null || bookName.isEmpty()){
            throw new InvalidArgumentsException();
        }

        return this.bookRepository.findByBookName(bookName);
    }

    //in bookRepository
    //TODO: listBooksByNameAndGenre, listBooksByNameAndGenreAndWriter, listBooksByNameAndWriter
    //TODO: listBooksByGenre, listBooksByWriter
}
